package com.iwown.sample4GApi.service;

import com.google.protobuf.ByteString;
import com.google.protobuf.InvalidProtocolBufferException;

import com.iwown.sample4GApi.protobuf.*;
import com.iwown.sample4GApi.utils.ByteUtils;
import com.iwown.sample4GApi.utils.MyDateTimeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class HistoryDataParser implements IDataParser {
    private static final Logger log = LoggerFactory.getLogger(HistoryDataParser.class);
    @Override
    public void DataParse(byte[] payload) {
        HisDataOuterClass.HisNotification hisNotify = null;
        try {
            hisNotify = HisDataOuterClass.HisNotification.parseFrom(payload);
        } catch (InvalidProtocolBufferException ex) {
            ex.printStackTrace();
            return;
        }
        HisDataOuterClass.HisDataType dataType = hisNotify.getType();
        //index table can be ignored at present, it's the index of history data
        if(hisNotify.hasIndexTable()){
            HisDataOuterClass.HisIndexTable idxTable = hisNotify.getIndexTable();
            if(idxTable.getIndexList() != null && idxTable.getIndexList().size()>0){
                idxTable.getIndexList().stream().forEach(
                        idx -> {
                            String dt_str = MyDateTimeUtils.convert2DateTimeString(idx.getTime().getSeconds());
                            log.info(String.format("index table time:%s, seq:%d-%d",dt_str,idx.getStartSeq(),idx.getEndSeq()));
                        }
                );
            }
        }
        if(hisNotify.hasHisData()){
            HisDataOuterClass.HisData hisData = hisNotify.getHisData();
            //health data
            if(hisData.hasHealth()){
                HisHealthData.HisDataHealth healthData = hisData.getHealth();
                parseHealthData(healthData);
            }
            //ecg data
            if(hisData.hasEcg()){
                HisEcgData.HisDataECG ecgData = hisData.getEcg();
                parseEcgData(ecgData);
            }
            //rri data
            if(hisData.hasRri()){
                HisRriData.HisDataRRI rriData = hisData.getRri();
                parseRriData(rriData);
            }
            //third party device data
            if(hisData.hasThirdPartyData()){
                ThirdPartyHealthData.HisDataThirdParty thirdParty = hisData.getThirdPartyData();
                ThirdPartyHealthData.ThirdPartyDataHealth thirdPartyData = thirdParty.getDataHealth();
                parseThirdPartyData(thirdPartyData);
            }
            //continuous spo2 data
            if(hisData.hasSpo2()){
                HisSpo2Data.HisDataSpo2 spo2Data = hisData.getSpo2();
                parseSpo2Data(spo2Data);
            }
            //ppg data
            if(hisData.hasPpg()){
                HisPpgData.HisDataPPG ppgData = hisData.getPpg();
                parsePpgData(ppgData);
            }
            //gensor data
            if(hisData.hasACCelerometerData()){
                ACCelerometerData.HisACCelerometer accData = hisData.getACCelerometerData();
                parseAccelerometerData(accData);
            }
            //multi-leads ecg
            if(hisData.hasMultiLeadsECG()){
                HisMultiLeadsEcg.HisDataMultiLeadsECG multiLeadsEcg = hisData.getMultiLeadsECG();
                parseMultiLeadsEcgData(multiLeadsEcg);
            }
        }
    }

    private void parseHealthData(HisHealthData.HisDataHealth healthData){
        String data_time = MyDateTimeUtils.convert2DateTimeString(healthData.getTimeStamp().getDateTime().getSeconds());
        //sport step/distance/calorie
        if(healthData.hasPedoData()){
            HisHealthData.HisHealthPedo his_pedo = healthData.getPedoData();
            float distance = his_pedo.getDistance()*0.1F;
            float calorie = his_pedo.getCalorie()*0.1F;
            log.info(String.format("%s step-%d, distance-%f, calorie-%f",data_time,his_pedo.getStep(),distance,calorie));
        }
        //heart rate
        if(healthData.hasHrData()){
            HisHealthData.HisHealthHr his_hr = healthData.getHrData();
            log.info(String.format("%s avg bpm-%d, max bpm-%d, min bpm-%d",data_time,his_hr.getAvgBpm(),his_hr.getMaxBpm(),his_hr.getMinBpm()));
        }
        //spo2
        if(healthData.hasBxoyData()){
            HisHealthData.HisHealthBOxy his_boxy = healthData.getBxoyData();
            log.info(String.format("%s avg boxy-%d, max boxy-%d, min boxy-%d",data_time,his_boxy.getAgvOxy(),his_boxy.getMaxOxy(),his_boxy.getMinOxy()));
        }
        //blood pressure
        if(healthData.hasBpData()){
            HisHealthData.HisHealthBp his_bp = healthData.getBpData();
            log.info(String.format("%s sbp-%d, dbp-%d",data_time,his_bp.getSbp(),his_bp.getDbp()));
        }
        //fatigue data
        if(healthData.hasHrvData()){
            HisHealthData.HisHealthHrv his_hrv = healthData.getHrvData();
            int fatigue = (int)his_hrv.getFatigue();
            if(fatigue<=0){
                fatigue = (int)(Math.log((double)his_hrv.getRMSSD())*20);
            }
            log.info(String.format("%s fatigue-%d",data_time,fatigue));
        }
        //temperature
        if(healthData.hasTemperatureData()){
            HisHealthData.HisHealthTemp his_temp = healthData.getTemperatureData();
            double shellTemp = 1.0* ByteUtils.loword(his_temp.getEviBody())/100.0;
            double envTemp = 1.0*ByteUtils.hiword(his_temp.getEviBody())/100.0;
            double axillaryTemp = 1.0*ByteUtils.loword(his_temp.getEstiArm())/100.0;
            double estTemp = 1.0*ByteUtils.hiword(his_temp.getEstiArm())/100.0;
            log.info(String.format("%s env-%f, est-%f, axillary-%f, shell-%f",data_time,envTemp,estTemp,axillaryTemp,shellTemp));
        }
        if(healthData.hasSleepData()){
            HisHealthData.HisHealthSleep sleepData = healthData.getSleepData();
            List<Integer> dataList = sleepData.getSleepDataList();
            log.info(String.format("sleep charget:%b,shutdown:%b,data count:%d",sleepData.getCharge(),sleepData.getShutDown(),dataList.size()));
        }
        //bp hr
        if(healthData.hasBpBpmData()){
            HisHealthData.HisHealthbp_bpm bpBpmData = healthData.getBpBpmData();
            int bp_hr = bpBpmData.getBpm();
            log.info(String.format("bp hr:%d",bp_hr));
        }
    }

    private void parseEcgData(HisEcgData.HisDataECG ecgData){
        String data_time = MyDateTimeUtils.convert2DateTimeString(ecgData.getTimeStamp().getDateTime().getSeconds());
        List<Integer> rawDataList = ecgData.getRawDataList();
        log.info(String.format("ecg time:%s, count:%d",data_time,rawDataList.size()));
    }

    private void parseRriData(HisRriData.HisDataRRI rriData){
        String data_time = MyDateTimeUtils.convert2DateTimeString(rriData.getTimeStamp().getDateTime().getSeconds());
        List<Integer> rawDataList = rriData.getRawDataList();
        List<Short> actualDataList = new ArrayList<>();
        for(int i=0;i<rawDataList.size();i++){
            short fval = (short)((rawDataList.get(i) >> 16) & 0x0000ffff);
            short sval = (short)(rawDataList.get(i) & 0x0000ffff);
            actualDataList.add(fval);
            actualDataList.add(sval);
        }
        log.info(String.format("rri time:%s, count:%d",data_time,actualDataList.size()));
    }

    private void parseThirdPartyData(ThirdPartyHealthData.ThirdPartyDataHealth thirdPartyData){
        String macAddr = thirdPartyData.getMacAddr();
        //bp data
        if(thirdPartyData.hasBpData()){
            ThirdPartyHealthData.ThirdPartyHealthBp bpData = thirdPartyData.getBpData();
            String data_time = MyDateTimeUtils.convert2DateTimeString(bpData.getTime().getDateTime().getSeconds());
            //mode 0 manual, 1 auto
            log.info(String.format("3rd party bp time:%s, sbp:%d, dbp:%d, hr:%d, pulse:%d, mode:%d",
                    data_time,bpData.getSbp(),bpData.getDbp(),bpData.getHr(),bpData.getPulse(),
                    bpData.getMODE().ordinal()));
        }
        //glu data
        if(thirdPartyData.hasGluData()){
            ThirdPartyHealthData.ThirdPartyHealthGlu gluData = thirdPartyData.getGluData();
            String data_time = MyDateTimeUtils.convert2DateTimeString(gluData.getTime().getDateTime().getSeconds());
            log.info(String.format("3rd party bloodglucose time:%s, val:%d",data_time,gluData.getGlu()));
        }
        //scale data
        if(thirdPartyData.hasScaleData()){
            ThirdPartyHealthData.ThirdPartyHealthScale scaleData = thirdPartyData.getScaleData();
            String data_time = MyDateTimeUtils.convert2DateTimeString(scaleData.getTime().getDateTime().getSeconds());
            log.info(String.format("3rd party scale time:%s, weight:%d, impedance:%d, uints:%d, body_fat_percentage:%d",
                    data_time,scaleData.getWeight(),
                    scaleData.getImpedance(),scaleData.getUints(),
                    scaleData.getBodyFatPercentage()));
        }
        //spo2 data
        if(thirdPartyData.hasSpo2Data()){
            ThirdPartyHealthData.ThirdPartyHealthSpo2 spo2Data = thirdPartyData.getSpo2Data();
            String data_time = MyDateTimeUtils.convert2DateTimeString(spo2Data.getTime().getDateTime().getSeconds());
            log.info(String.format("3rd party spo2 time:%s, spo2:%d, bpm:%d, pi:%d",
                    data_time,spo2Data.getSpo2(),spo2Data.getBpm(),spo2Data.getPi()));
        }
        //temperature data
        if(thirdPartyData.hasTempData()){
            ThirdPartyHealthData.ThirdPartyHealthTemp tempData = thirdPartyData.getTempData();
            String data_time = MyDateTimeUtils.convert2DateTimeString(tempData.getTime().getDateTime().getSeconds());
            log.info(String.format("3rd party temperature time:%s, val:%d",
                    data_time,tempData.getBodyTemp()));
        }
        //blood ketones
        if(thirdPartyData.hasBloodKetonesData()){
            ThirdPartyHealthData.ThirdPartyHealthBloodKetones bkData = thirdPartyData.getBloodKetonesData();
            String data_time = MyDateTimeUtils.convert2DateTimeString(bkData.getTime().getDateTime().getSeconds());
            log.info(String.format("3rd party blood ketones time:%s, val:%d",
                    data_time,bkData.getBloodKetones()));
        }
        //uric acid
        if(thirdPartyData.hasUricAcidData()){
            ThirdPartyHealthData.ThirdPartyHealthUricAcid uaData = thirdPartyData.getUricAcidData();
            String data_time = MyDateTimeUtils.convert2DateTimeString(uaData.getTime().getDateTime().getSeconds());
            log.info(String.format("3rd party uric acid time:%s, val:%d",
                    data_time,uaData.getUricAcid()));
        }
    }

    private void parseSpo2Data(HisSpo2Data.HisDataSpo2 spo2Data) {
        String data_time = MyDateTimeUtils.convert2DateTimeString(spo2Data.getTimeStamp().getDateTime().getSeconds());
        List<Integer> spo2List = spo2Data.getSpo2DataList();
        log.info(String.format("spo2 history time: %s",data_time));
        for(int i=0;i<spo2List.size();i++){
            int value = spo2List.get(i);
            int spo2 = (value >> 24) & 0xFF;
            int hr = (value >> 16) & 0xFF;
            int perfusion = (value >> 8) & 0xFF;
            int touch = (value) & 0xFF;
            log.info(String.format("spo2: %d, hr: %d, perfusion: %d, touch: %d",
                    spo2,hr,perfusion,touch));
        }
    }

    private void parsePpgData(HisPpgData.HisDataPPG ppgData) {
        String data_time = MyDateTimeUtils.convert2DateTimeString(ppgData.getTimeStamp().getDateTime().getSeconds());
        List<Integer> rawDataList = ppgData.getRawDataList();
        log.info(String.format("ppg history time: %s",data_time));
        List<Short> actualDataList = new ArrayList<>();
        for(int i=0;i<rawDataList.size();i++){
            short fval = (short)((rawDataList.get(i) >> 16) & 0x0000ffff);
            if(fval<0){
                fval=(short)(fval*-1);
            }
            short sval = (short)(rawDataList.get(i) & 0x0000ffff);
            if(sval<0){
                sval=(short)(sval*-1);
            }
            actualDataList.add(fval);
            actualDataList.add(sval);
            log.info(String.format("%s ppg:%d,%d",data_time,fval,sval));
        }
    }

    private void parseAccelerometerData(ACCelerometerData.HisACCelerometer accData) {
        String data_time = MyDateTimeUtils.convert2DateTimeString(accData.getTimeStamp().getDateTime().getSeconds());
        ByteString xBstr = accData.getAccX();
        List<Integer> xList = parseByteString(xBstr);

        ByteString yBstr = accData.getAccY();
        List<Integer> yList = parseByteString(yBstr);

        ByteString zBstr = accData.getAccZ();
        List<Integer> zList = parseByteString(zBstr);
        for(int i=0;i<xList.size();i++){
            log.info(String.format("%s acc x:%d,y:%d,z:%d",data_time,xList.get(i),yList.get(i),zList.get(i)));
        }
    }

    private void parseMultiLeadsEcgData(HisMultiLeadsEcg.HisDataMultiLeadsECG multiLeadsEcg){
        String data_time = MyDateTimeUtils.convert2DateTimeString(multiLeadsEcg.getTimeStamp().getDateTime().getSeconds());
        int channel = multiLeadsEcg.getNumberOfChannels();
        int single_channel_len = multiLeadsEcg.getSingleDataByteLen();
        log.info(String.format("%s channel:%d, single channel length:%d",data_time,channel,single_channel_len));
        ByteString aBstr = multiLeadsEcg.getRawData();
        byte[] buffer = aBstr.toByteArray();
        int unit_size = single_channel_len*channel;
        for(int i=unit_size;i<=buffer.length;i=i+unit_size){
            for(int j=0;j<channel;j++){
                int num=0;
                for(int k=0;k<single_channel_len;k++){
                    int pos = i-single_channel_len*(channel-j)+k;
                    int offset = 8*(single_channel_len-k-1);
                    int part = (int)(buffer[pos]<<offset);
                    num += part;
                }
                log.info(String.format("channel %d, val:%d",j+1,num));
            }
        }
    }

    private List<Integer> parseByteString(ByteString bstr){
        List<Integer> xList = new ArrayList<>();
        byte[] xbuffer = bstr.toByteArray();
        for(int j=1;j<xbuffer.length;j=j+2){
            byte low = (byte)xbuffer[j-1];
            int high = (int)(xbuffer[j]<<8);
            int real = low+high;
            xList.add(real);
        }
        return xList;
    }


}
